import 'dart:async';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter_lock_new_screen/src/mycolors.dart';

typedef DeleteCode = void Function();
typedef PassCodeVerify = Future<bool> Function(List<int> passcode);

class LockScreen extends StatefulWidget {
  /// Password on success method
  final VoidCallback onSuccess;

  /// Password finger function for auth
  final VoidCallback? fingerFunction;

  /// Password finger verify for auth
  final bool? fingerVerify;

  /// screen title
  final String title;

  /// Pass length
  final int passLength;

  /// Wrong password dialog
  final bool? showWrongPassDialog;

  /// Showing finger print area
  final bool? showFingerPass;

  /// Wrong password dialog title
  final String? wrongPassTitle;

  /// Wrong password dialog content
  final String? wrongPassContent;

  /// Wrong password dialog button text
  final String? wrongPassCancelButtonText;

  /// Background image
  final String? bgImage;

  //Logo

  final String? logo;

  /// Color for numbers
  final Color? numColor;

  /// Finger print image
  final Widget? fingerPrintImage;

  /// border color
  final Color? borderColor;

  /// foreground color
  final Color? foregroundColor;

  /// Password verify
  final PassCodeVerify passCodeVerify;

  /// Lock Screen constructer
  LockScreen({
    required this.onSuccess,
    required this.title,
    this.borderColor,
    this.foregroundColor = Colors.transparent,
    required this.passLength,
    required this.passCodeVerify,
    this.fingerFunction,
    this.fingerVerify = false,
    this.showFingerPass = false,
    this.bgImage,
    this.logo,
    this.numColor = Colors.black,
    this.fingerPrintImage,
    this.showWrongPassDialog = true,
    this.wrongPassTitle,
    this.wrongPassContent,
    this.wrongPassCancelButtonText,
  })  : assert(passLength <= 8),
        assert(bgImage != null),
        assert(borderColor != null),
        assert(foregroundColor != null);

  @override
  _LockScreenState createState() => _LockScreenState();
}

class _LockScreenState extends State<LockScreen> {
  var _currentCodeLength = 0;
  var _inputCodes = <int>[];
  var _currentState = 0;
  Color circleColor = Colors.white;

  _onCodeClick(int code) {
    if (_currentCodeLength < widget.passLength) {
      setState(() {
        _currentCodeLength++;
        _inputCodes.add(code);
      });

      if (_currentCodeLength == widget.passLength) {
        widget.passCodeVerify(_inputCodes).then((onValue) {
          if (onValue) {
            setState(() {
              _currentState = 1;
            });
            widget.onSuccess();
          } else {
            _currentState = 2;
            Timer(Duration(milliseconds: 1000), () {
              setState(() {
                _currentState = 0;
                _currentCodeLength = 0;
                _inputCodes.clear();
              });
            });
            if (widget.showWrongPassDialog!) {
              showDialog(
                  barrierDismissible: false,
                  context: context,
                  builder: (BuildContext context) {
                    return Center(
                      child: AlertDialog(
                        title: Text(
                          widget.wrongPassTitle!,
                          style: const TextStyle(fontFamily: "Open Sans"),
                        ),
                        content: Text(
                          widget.wrongPassContent!,
                          style: const TextStyle(fontFamily: "Open Sans"),
                        ),
                        actions: <Widget>[
                          TextButton(
                            onPressed: () => Navigator.pop(context),
                            child: Text(
                              widget.wrongPassCancelButtonText!,
                              style: const TextStyle(color: Colors.blue),
                            ),
                          )
                        ],
                      ),
                    );
                  });
            }
          }
        });
      }
    }
  }

  _fingerPrint() {
    if (widget.fingerVerify!) {
      widget.onSuccess();
    }
  }

  _deleteCode() {
    setState(() {
      if (_currentCodeLength > 0) {
        _currentState = 0;
        _currentCodeLength--;
        _inputCodes.removeAt(_currentCodeLength);
      }
    });
  }

  _deleteAllCode() {
    setState(() {
      if (_currentCodeLength > 0) {
        _currentState = 0;
        _currentCodeLength = 0;
        _inputCodes.clear();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    Future.delayed(const Duration(milliseconds: 200), () {
      _fingerPrint();
    });
    return Scaffold(
      backgroundColor: Colors.white,
      body: Stack(
        children: <Widget>[
          Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Expanded(
                flex: Platform.isIOS ? 2 : 2,
                child: Container(
                  width: double.infinity,
                  padding: const EdgeInsets.only(left: 0, top: 0),
                  decoration: const BoxDecoration(),
                  child: Stack(
                    children: <Widget>[
                      ClipPath(
                        child: Container(
                          height: MediaQuery.of(context).size.height,
                          width: MediaQuery.of(context).size.width,
                          decoration: BoxDecoration(
                            image: DecorationImage(
                              image: AssetImage(widget.bgImage!),
                              fit: BoxFit.cover,
                              colorFilter: ColorFilter.mode(
                                Colors.grey.shade800,
                                BlendMode.hardLight,
                              ),
                            ),
                          ),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.start,
                            children: <Widget>[
                              SizedBox(
                                height: Platform.isIOS ? 15 : 15,
                              ),
                              Image.asset(
                                widget.logo!,
                                width: 200,
                                height: 158,
                              ),
                              SizedBox(
                                height: Platform.isIOS ? 15 : 15,
                              ),
                              Text(
                                widget.title,
                                style: const TextStyle(
                                    color: Colors.white,
                                    fontSize: 20,
                                    fontWeight: FontWeight.bold,
                                    fontFamily: "Open Sans"),
                              ),
                              SizedBox(
                                height: Platform.isIOS ? 50 : 50,
                              ),
                              const Text(
                                "Şifrənizi daxil edin",
                                style: TextStyle(
                                    color: Colors.white,
                                    fontSize: 18,
                                    fontFamily: "Open Sans"),
                              ),
                              SizedBox(
                                height: Platform.isIOS ? 30 : 15,
                              ),
                              CodePanel(
                                codeLength: widget.passLength,
                                currentLength: _currentCodeLength,
                                borderColor: widget.borderColor,
                                foregroundColor: widget.foregroundColor,
                                deleteCode: _deleteCode,
                                fingerVerify: widget.fingerVerify!,
                                status: _currentState,
                              ),
                              SizedBox(
                                height: Platform.isIOS ? 30 : 5,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
              Expanded(
                flex: Platform.isIOS ? 2 : 2,
                child: Container(
                  padding: const EdgeInsets.only(left: 0, top: 15),
                  child: NotificationListener<OverscrollIndicatorNotification>(
                    onNotification: (overscroll) {
                      overscroll.disallowIndicator();
                      return true;
                    },
                    child: GridView.count(
                      crossAxisCount: 3,
                      childAspectRatio: 3.2,
                      mainAxisSpacing: 35,
                      padding: const EdgeInsets.all(10),
                      children: <Widget>[
                        buildContainerCircle(1),
                        buildContainerCircle(2),
                        buildContainerCircle(3),
                        buildContainerCircle(4),
                        buildContainerCircle(5),
                        buildContainerCircle(6),
                        buildContainerCircle(7),
                        buildContainerCircle(8),
                        buildContainerCircle(9),
                        widget.showFingerPass!
                            ? Positioned(
                                top: MediaQuery.of(context).size.height /
                                    (Platform.isIOS ? 4 : 5),
                                left: 20,
                                bottom: 10,
                                child: GestureDetector(
                                  onTap: () {
                                    widget.fingerFunction!();
                                  },
                                  child: widget.fingerPrintImage!,
                                ),
                              )
                            : Container(),
                        buildContainerCircle(0),
                        buildContainerIcon(Icons.backspace_outlined),
                        const SizedBox(height: 0),
                        TextButton(
                          style: TextButton.styleFrom(
                            textStyle: const TextStyle(
                                fontSize: 11,
                                decoration: TextDecoration.underline),
                          ),
                          onPressed: () {},
                          child: const Text('Şifrəni unutmusunuz?'),
                        ),
                      ],
                    ),
                  ),
                ),
              )
            ],
          ),
        ],
      ),
    );
  }

  Widget buildContainerCircle(int number) {
    return InkResponse(
      highlightColor: Colors.red,
      onTap: () {
        _onCodeClick(number);
      },
      child: Container(
        height: 50,
        width: 50,
        decoration: const BoxDecoration(
            color: MyColors.duymerengi,
            shape: BoxShape.circle,
            boxShadow: [
              BoxShadow(
                color: Colors.black12,
                blurRadius: 10,
                spreadRadius: 0,
              )
            ]),
        child: Center(
          child: Text(
            number.toString(),
            style: TextStyle(
                fontSize: 28,
                fontWeight: FontWeight.normal,
                color: widget.numColor),
          ),
        ),
      ),
    );
  }

  Widget buildRemoveIcon(IconData icon) {
    return InkResponse(
      onTap: () {
        if (0 < _currentCodeLength) {
          _deleteAllCode();
        }
      },
      child: Container(
        height: 50,
        width: 50,
        decoration: const BoxDecoration(
            color: Colors.white,
            shape: BoxShape.circle,
            boxShadow: [
              BoxShadow(
                color: Colors.black12,
                blurRadius: 10,
                spreadRadius: 0,
              )
            ]),
        child: Center(
          child: Icon(
            icon,
            size: 30,
            color: widget.numColor,
          ),
        ),
      ),
    );
  }

  Widget buildContainerIcon(IconData icon) {
    return InkResponse(
      onTap: () {
        if (0 < _currentCodeLength) {
          setState(() {
            circleColor = Colors.white;
          });
          Future.delayed(const Duration(milliseconds: 200)).then((func) {
            setState(() {
              circleColor = Colors.white;
            });
          });
        }
        _deleteCode();
      },
      child: Container(
        height: 50,
        width: 50,
        decoration: BoxDecoration(
          color: circleColor,
          shape: BoxShape.circle,
        ),
        child: Center(
          child: Icon(
            icon,
            size: 30,
            color: widget.numColor,
          ),
        ),
      ),
    );
  }
}

class CodePanel extends StatelessWidget {
  final codeLength;
  final currentLength;
  final borderColor;
  final bool? fingerVerify;
  final foregroundColor;
  final H = 50.0;
  final W = 50.0;
  final DeleteCode? deleteCode;
  final int? status;
  CodePanel(
      {this.codeLength,
      this.currentLength,
      this.borderColor,
      this.foregroundColor,
      this.deleteCode,
      this.fingerVerify,
      this.status})
      : assert(codeLength > 0),
        assert(currentLength >= 0),
        assert(currentLength <= codeLength),
        assert(deleteCode != null),
        assert(status == 0 || status == 1 || status == 2);

  @override
  Widget build(BuildContext context) {
    var circles = <Widget>[];
    var color = borderColor;
    int circlePice = 1;

    if (fingerVerify == true) {
      do {
        circles.add(
          SizedBox(
            width: W,
            height: H,
            child: Container(
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: color, width: 1.0),
                color: Colors.green.shade500,
              ),
            ),
          ),
        );
        circlePice++;
      } while (circlePice <= codeLength);
    } else {
      if (status == 1) {
        color = Colors.green.shade500;
      }
      if (status == 2) {
        color = Colors.red.shade500;
      }
      for (int i = 1; i <= codeLength; i++) {
        if (i > currentLength) {
          circles.add(SizedBox(
              width: 40,
              height: 70,
              child: Container(
                decoration: BoxDecoration(
                    shape: BoxShape.rectangle,
                    borderRadius: BorderRadius.circular(5),
                    border: Border.all(color: color, width: 2.0),
                    color: Colors.white),
              )));
        } else {
          circles.add(SizedBox(
              width: 40,
              height: 70,
              child: Container(
                decoration: BoxDecoration(
                  shape: BoxShape.rectangle,
                  borderRadius: BorderRadius.circular(5),
                  border: Border.all(color: color, width: 10),
                  color: MyColors.grey_60,
                ),
              )));
        }
      }
    }

    return SizedBox.fromSize(
      size: Size(MediaQuery.of(context).size.width, 30.0),
      child:
          Row(mainAxisAlignment: MainAxisAlignment.center, children: <Widget>[
        SizedBox.fromSize(
            size: Size(50.0 * codeLength, H),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: circles,
            )),
      ]),
    );
  }
}

class BgClipper extends CustomClipper<Path> {
  @override
  Path getClip(Size size) {
    Path path = Path();
    path.lineTo(0, size.height);
    path.lineTo(size.width, size.height / 1.5);
    path.lineTo(size.width, 0);
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return false;
  }
}
